@@ -292,6 +292,9 @@ write_data2 <- function(
   endRow <- (startRow - 1) + data_nrow
   endCol <- (startCol - 1) + data_ncol
 
+  if (endRow > 1048576 || endCol > 16384)
+    stop("Dimensions exceed worksheet")
+
   dims <- paste0(
     int2col(startCol), startRow,
     ":",
@@ -344,7 +347,7 @@ write_data2 <- function(
 
   # rtyp character vector per row
   # list(c("A1, ..., "k1"), ...,  c("An", ..., "kn"))
-  rtyp <- dims_to_dataframe(dims, fill = TRUE)
+  rtyp <- dims_to_dataframe(dims, fill = FALSE)
 
   rows_attr <- vector("list", data_nrow)
 
@@ -407,7 +410,7 @@ write_data2 <- function(
 
   # if rownames = TRUE and data_table = FALSE, remove "_rownames_"
   if (!data_table && rowNames && colNames) {
-    cc <- cc[cc$r != rtyp[1, 1], ]
+    cc <- cc[cc$r != paste0(names(rtyp)[1], rownames(rtyp)[1]), ]
   }
 
   if (is.null(wb$worksheets[[sheetno]]$sheet_data$cc)) {
@@ -536,6 +539,19 @@ write_data2 <- function(
         numfmt = numfmt_hms
       )
     }
+    if (any(dc == openxlsx2_celltype[["currency"]])) { # currency
+      numfmt_currency <- getOption("openxlsx2.currencyFormat", default = 44)
+      ## For vignette: Builtin style for USD
+      #"_-[$$-409]* #,##0.00_ ;_-[$$-409]* \\-#,##0.00\\ ;_-[$$-409]* &quot;-&quot;??_ ;_-@_ "
+
+      dim_sel <- get_data_class_dims("currency")
+      # message("currency: ", dim_sel)
+
+      wb$add_numfmt(
+        dim = dim_sel,
+        numfmt = numfmt_currency
+      )
+    }
     if (any(dc == openxlsx2_celltype[["accounting"]])) { # accounting
       numfmt_accounting <- getOption("openxlsx2.accountingFormat", default = 4)
 
@@ -624,7 +640,7 @@ write_data2 <- function(
     xml <- wb$calcChain
     calcChainR <- rbindlist(xml_attr(xml, "calcChain", "c"))
     # according to the documentation there can be cases, without the sheetno reference
-    sel <- calcChainR$r %in% rtyp & calcChainR$i == sheetno
+    sel <- calcChainR$r %in% wb$worksheets[[sheetno]]$sheet_data$cc$r & calcChainR$i == sheetno
     rmCalcChain <- as.integer(rownames(calcChainR[sel, , drop = FALSE]))
     if (length(rmCalcChain)) {
       xml <- xml_rm_child(xml, xml_child = "c", which = rmCalcChain)
@@ -759,7 +775,7 @@ write_data_table <- function(
   if (applyCellStyle) {
     if (is.null(dim(x))) {
       is_hyperlink <- inherits(x, "hyperlink")
-    } else {
+    } else if (is.data.frame(x)) { # dont check on a matrix
       is_hyperlink <- vapply(x, inherits, what = "hyperlink", FALSE)
     }
 
